
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include <map>

#include "Dispatcher.hh"


namespace Mf {


Notification::~Notification() {}


class Dispatcher::DispatcherImpl
{
public:
	DispatcherImpl() : id(1) {}

	Dispatcher::Handler getNewHandlerID()
	{
		id += 4;
		return (Dispatcher::Handler)id;
	}

	typedef std::pair<Dispatcher::Handler,Dispatcher::Function>	Callback;
	typedef std::multimap<std::string,Callback>					CallbackLookup;
	typedef CallbackLookup::iterator							CallbackIter;

	typedef std::multimap<Dispatcher::Handler,std::string>		HandlerLookup;
	typedef HandlerLookup::iterator								HandlerIter;

	unsigned long long	id;

	CallbackLookup		callbacks;
	HandlerLookup		handlers;
};


Dispatcher::Dispatcher() :
	impl_(new Dispatcher::DispatcherImpl) {}


// TODO these methods are ugly

Dispatcher::Handler Dispatcher::addHandler(const std::string& message,
		const Function& callback)
{
	return addHandler(message, callback, impl_->getNewHandlerID());
}

Dispatcher::Handler Dispatcher::addHandler(const std::string& message,
		const Function& callback, Handler id)
{
	std::pair<std::string,Dispatcher::DispatcherImpl::Callback>
		callbackPair(message, Dispatcher::DispatcherImpl::Callback(id, callback));

	std::pair<Handler,std::string> handlerPair(id, message);

	impl_->callbacks.insert(callbackPair);
	impl_->handlers.insert(handlerPair);

	return id;
}


void Dispatcher::removeHandler(Handler id)
{
	std::pair<Dispatcher::DispatcherImpl::HandlerIter,Dispatcher::DispatcherImpl::HandlerIter>
		handlers(impl_->handlers.equal_range(id));

	Dispatcher::DispatcherImpl::HandlerIter it;
	for (it = handlers.first; it != handlers.second; ++it)
	{
		Dispatcher::DispatcherImpl::CallbackIter first = impl_->callbacks.find((*it).second);
		Dispatcher::DispatcherImpl::CallbackIter last = impl_->callbacks.end();

		Dispatcher::DispatcherImpl::CallbackIter jt;
		for (jt = first; jt != last; ++jt)
		{
			if (((*jt).second).first == id)
			{
				impl_->callbacks.erase(jt);
				break;
			}
		}
	}
	
	impl_->handlers.erase(id);
}


void Dispatcher::dispatch(const std::string& message)
{
	dispatch(message, Notification());
}

void Dispatcher::dispatch(const std::string& message, const Notification& param)
{
	std::pair<Dispatcher::DispatcherImpl::CallbackIter,Dispatcher::DispatcherImpl::CallbackIter>
		callbacks(impl_->callbacks.equal_range(message));

	Dispatcher::DispatcherImpl::CallbackIter it;
	for (it = callbacks.first; it != callbacks.second; ++it)
	{
		Function callback = ((*it).second).second;
		callback(param);
	}
}


} // namespace Mf

/** vim: set ts=4 sw=4 tw=80: *************************************************/

