
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_SETTINGS_HH_
#define _MOOF_SETTINGS_HH_

/**
 * @file Settings.hh
 * Load, store, save program settings.
 */

#include <map>
#include <string>

#include <Moof/Singleton.hh>
#include <Moof/Serializable.hh>


namespace Mf {


class Settings : public Singleton<Settings>
{
public:
	Settings() {}
	Settings(int argc, char* argv[]);

	void parseArgs(int argc, char* argv[]);

	void loadFromFile(const std::string& filePath, bool precedence = false);
	void loadFromFiles(const std::vector<std::string>& filePaths,
			bool precedence = false);

	template <typename T>
	bool get(const std::string& key, T& value);
	template <typename T>
	bool getNumber(const std::string& key, T& value);

private:
	std::map<std::string,SerializablePtr> map_;
};


template <typename T>
bool Settings::get(const std::string& key, T& value)
{
	std::map<std::string,SerializablePtr>::const_iterator it = map_.find(key);

	if (it != map_.end())
	{
		SerializablePtr obj = (*it).second;
		return obj->get(value);
	}
	else
	{
		return false;
	}
}

template <typename T>
bool Settings::getNumber(const std::string& key, T& value)
{
	std::map<std::string,SerializablePtr>::const_iterator it = map_.find(key);

	if (it != map_.end())
	{
		SerializablePtr obj = (*it).second;
		return obj->getNumber(value);
	}
	else
	{
		return false;
	}
}


} // namepsace Mf

#endif // _MOOF_SETTINGS_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

