
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _MOOF_TREE_HH_
#define _MOOF_TREE_HH_

#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
	

namespace Mf {


template <typename T>
class Tree
{
public:
	typedef boost::shared_ptr<Tree> Ptr;
	typedef boost::weak_ptr<Tree> WeakPtr;

private:
	WeakPtr	parent_;
	WeakPtr	prevSibling_;
	Ptr		next_;
	WeakPtr	lastDescendant_;

public:

	T		data;

	Tree() {}
	Tree(const T& item) :
		data(item) {}

	inline Ptr next() const
	{
		return next_;
	}

	inline Ptr prev() const
	{
		Ptr parent = parent_.lock();

		if (parent)
		{
			if (parent->next_.get() == this)
			{
				return parent;
			}
			else
			{
				return prevSibling_.lock()->lastDescendant_.lock();
			}
		}

		return Ptr();
	}

	inline Ptr firstChild() const
	{
		if (next_ && next_->parent_.lock().get() == this)
		{
			return next_;
		}
		
		return Ptr();
	}

	inline Ptr lastChild() const
	{
		Ptr child = firstChild();

		if (child)
		{
			child = child->prevSibling_.lock();
		}

		return child;
	}

	inline Ptr nextSibling() const
	{
		Ptr sibling = lastDescendant_.lock()->next_;

		if (sibling && sibling->parent_.lock() != parent_.lock())
		{
			return Ptr();
		}

		return sibling;
	}

	inline Ptr prevSibling() const
	{
		Ptr parent = parent_.lock();

		if (parent && parent->next_.get() != this)
		{
			return prevSibling_.lock();
		}

		return Ptr();
	}


	class Iterator
	{
		Ptr current;

	public:

	};


	void insert()
	{
	}

	void remove()
	{
	}

};


} // namespace Mf

#endif // _MOOF_TREE_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

